<?php
/** @noinspection PhpIncludeInspection */

namespace Boekuwzending\WooCommerce\Admin\Metabox;

use Boekuwzending\WooCommerce\Helper\Api;
use Boekuwzending\WooCommerce\Helper\Data;
use Boekuwzending\WooCommerce\Helper\Settings;
use Boekuwzending\WooCommerce\Plugin;

/**
 * Class Order
 */
class Order
{
    /**
     * @var Data
     */
    protected static $dataHelper;

    /**
     * @var Settings
     */
    protected static $settingsHelper;

    /**
     * @var Api
     */
    protected static $apiHelper;

    /**
     * Initialize
     *
     * @param Settings $settingsHelper
     * @param Api $apiHelper
     * @param Data $dataHelper
     */
    public static function init(Settings $settingsHelper, Api $apiHelper, Data $dataHelper): void
    {
        self::$dataHelper = $dataHelper;
        self::$settingsHelper = $settingsHelper;
        self::$apiHelper = $apiHelper;

        add_action( 'add_meta_boxes', [__CLASS__, 'add'] );
        add_action('wp_ajax_boekuwzending_admin_metabox_order_render', [__CLASS__, 'render']);
    }

    /**
     * Add custom metabox to WooCommerce order page
     */
    public static function add(): void
    {
        add_meta_box(
            'woocommerce-order-boekuwzending',
            __( 'Boekuwzending', 'boekuwzending-for-woocommerce' ),
            [__CLASS__, 'renderBase'],
            'shop_order',
            'side',
            'default'
        );
    }

    /**
     * Renders the general container for the metabox
     *
     * @noinspection PhpIncludeInspection
     */
    public static function renderBase(): void
    {
        // Render metabox view
        require_once Plugin::getTemplatePath('admin/order/edit/meta/render');

        // Render script template for add shipping method backbone modal
        require_once Plugin::getTemplatePath('admin/order/edit/modal/add-shipping-method');

        // Render script template for change shipping method backbone modal
        require_once Plugin::getTemplatePath('admin/order/edit/modal/change-shipping-method');

        require_once Plugin::getTemplatePath('admin/order/edit/modal/get-shipping-services');
    }

    /**
     * Renders the possible actions based on the specific requirements
     */
    public static function render(): void
    {
        $orderId = absint(wp_unslash($_POST['orderId']));

        if (empty($orderId)) {
            return;
        }

        $order = wc_get_order($orderId);

        if (!$order) {
            return;
        }

        $testMode = self::$settingsHelper->isTestModeEnabled();

        if (false === self::$apiHelper->isIntegrated($testMode)) {
            require_once Plugin::getTemplatePath('admin/order/edit/meta/not_connected');

            die();
        }

        $shippingMethod = boekuwzendingGetShippingMethodForOrder($order);
        $shipments = self::$dataHelper->getBoekuwzendingShipments($order->get_id());

        /** @noinspection PhpUnusedLocalVariableInspection */
        $shipmentData = self::$dataHelper->getBoekuwzendingData($order->get_id());

        /** @noinspection PhpUnusedLocalVariableInspection */
        $showPickupPointInformation = $shippingMethod && wc_string_to_bool(
                $shippingMethod->get_meta('_pick_up')
            );
        /** @noinspection PhpUnusedLocalVariableInspection */
        $isNewOrder = wc_string_to_bool($_POST['isNewOrder']);

        if (!empty($shipments)) {
            require_once Plugin::getTemplatePath('admin/order/edit/meta/status');
        }

        $orderItems = $order->get_items( ['line_item'] );

        if (count($orderItems) > 0) {
            require_once Plugin::getTemplatePath('admin/order/edit/meta/actions');
        } else {
            require_once Plugin::getTemplatePath('admin/order/edit/meta/no_products');
        }

        die();
    }
}