<?php

namespace Boekuwzending\WooCommerce\Helper;

use Boekuwzending\Client;
use Boekuwzending\ClientFactory;
use Boekuwzending\WooCommerce\Plugin;
use RuntimeException;

/**
 * Class Api
 */
class Api
{
    /**
     * @var Client
     */
    protected static $apiClient;

    /**
     * @var Settings
     */
    protected $settingsHelper;

    /**
     * @param Settings $settingsHelper
     */
    public function __construct(Settings $settingsHelper)
    {
        $this->settingsHelper = $settingsHelper;
    }

    /**
     * @param bool $testMode
     * @return Client
     * @throws RuntimeException
     */
    public function getApiClient($testMode = false): Client
    {
        global $wp_version;

        $apiCredentials = $this->settingsHelper->getClientCredentials($testMode);

        if (has_filter('boekuwzending_api_credentials_filter')) {
            $apiCredentials = apply_filters('boekuwzending_api_credentials_filter', $apiCredentials);
        }

        if (empty($apiCredentials['clientId'])) {
            throw new RuntimeException(
                __(
                    'No Client Id provided. Please set your Boekuwzending API Client Id',
                    'boekuwzending-for-woocommerce'
                )
            );
        }

        if (empty($apiCredentials['clientSecret'])) {
            throw new RuntimeException(
                __(
                    'No Client Secret provided. Please set your Boekuwzending API Client Secret',
                    'boekuwzending-for-woocommerce'
                )
            );
        }

        if (empty(self::$apiClient)) {
            $client = ClientFactory::build(
                $apiCredentials['clientId'],
                $apiCredentials['clientSecret'],
                $apiCredentials['environment']
            );

            $client->addAdditionalUserAgent('WordPress/' . $wp_version ?? 'Unknown');
            $client->addAdditionalUserAgent('WooCommerce/' . get_option('woocommerce_version', 'Unknown'));
            $client->addAdditionalUserAgent('BoekuwzendingWoo/' . Plugin::PLUGIN_VERSION);

            self::$apiClient = $client;
        }

        return self::$apiClient;
    }

    /**
     * @param bool $testMode
     * @return bool
     */
    public function isIntegrated(bool $testMode = false): bool
    {
        $apiCredentials = $this->settingsHelper->getClientCredentials($testMode);

        return !(empty($apiCredentials['clientId']) || empty($apiCredentials['clientSecret']));
    }
}
